# ネ申エクセルについて
Excelで印刷物での見栄えを優先させて作成された表がしばしば見受けられます。
これらのExcelワークブックは、ネ申エクセル(神エクセル）と呼ばれたりします。
神エクセルは印刷物を作成すること（だけ）を念頭に置かれて作成されるため、
そこに含まれたデータを再利用する際に、余分な手間が必要となり、データの効率的な利用の障害となっています。

科研費申請書で有名になった方眼紙エクセルは極端な例ですが、
セルの結合を多用しているため、標準的な手法ではその中のデータを簡単にはプログラムに取り込めない
ということはしばしばあります。

一例として、首相官邸ホームページに掲載されている都道府県別のコロナワクチン接種状況のデータを見て見ましょう。

- 都道府県別の実績　https://www.kantei.go.jp/jp/content/kenbetsu-vaccination_data2.xlsx

この表には、都道府県別に、一般接種および医療対象者の二つの接種対象毎に表が作成されています。
また、ワクチンの種別毎に1回目としての接種回数と2回目としての接種回数がデータにふくまれています。
一般接種と医療対象者等のデータはそれぞれ別ワークシートに収められており、
テーブルの形式もそれぞれで若干の違いがあります。　
それらのテーブルでは、各列の項目名の行はセルの結合を組み合わせて作られており、
データを取り出す為には、worksheet毎にデータの処理を行う必要があります。


```python
import pandas
from urllib.request import urlopen
import openpyxl
from io import StringIO,BytesIO
wb=openpyxl.load_workbook(
        BytesIO(
            urlopen("https://www.kantei.go.jp/jp/content/kenbetsu-vaccination_data2.xlsx").read()
        ))
df0=pandas.read_excel(
    urlopen("https://www.kantei.go.jp/jp/content/kenbetsu-vaccination_data2.xlsx").read(),
    header=2,sheet_name=0)
```


```python
df0.columns, df0["都道府県名"]
```




    (Index(['都道府県名', '接種回数', '内１回目', '内２回目', 'Unnamed: 4'], dtype='object'),
     0                                                    合計
     1                                                01 北海道
     2                                                02 青森県
     3                                                03 岩手県
     4                                                04 宮城県
     5                                                05 秋田県
     6                                                06 山形県
     7                                                07 福島県
     8                                                08 茨城県
     9                                                09 栃木県
     10                                               10 群馬県
     11                                               11 埼玉県
     12                                               12 千葉県
     13                                               13 東京都
     14                                              14 神奈川県
     15                                               15 新潟県
     16                                               16 富山県
     17                                               17 石川県
     18                                               18 福井県
     19                                               19 山梨県
     20                                               20 長野県
     21                                               21 岐阜県
     22                                               22 静岡県
     23                                               23 愛知県
     24                                               24 三重県
     25                                               25 滋賀県
     26                                               26 京都府
     27                                               27 大阪府
     28                                               28 兵庫県
     29                                               29 奈良県
     30                                              30 和歌山県
     31                                               31 鳥取県
     32                                               32 島根県
     33                                               33 岡山県
     34                                               34 広島県
     35                                               35 山口県
     36                                               36 徳島県
     37                                               37 香川県
     38                                               38 愛媛県
     39                                               39 高知県
     40                                               40 福岡県
     41                                               41 佐賀県
     42                                               42 長崎県
     43                                               43 熊本県
     44                                               44 大分県
     45                                               45 宮崎県
     46                                              46 鹿児島県
     47                                               47 沖縄県
     48                                                  NaN
     49                        注：接種回数は一般接種（高齢者含む）と医療従事者等の合計。
     50                注：一般接種（高齢者含む）はワクチン接種記録システム(VRS)への報告と、
     51       　　医療従事者等はワクチン接種円滑化システム（V-SYS）への報告を、公表日で集計したもの。
     52    注：公表日におけるデータの計上方法等の注釈については、以下を参照（https://www.k...
     Name: 都道府県名, dtype: object)



このように、データ以外の情報　（ここでは"注”）もテーブル中にあり、プログラムでのデータ処理の例外扱いを増やしています。（注などは、テキストボックスや、フッターにいれることを検討して欲しいところです。ただ、フッターに入れることのできる文の長さには制限がある。~~テキストボックスはテーブルの長さが変わったときに位置が追随しないという問題はあります。~~ テキストボックスはプロパティの”セルに合わせて移動”を選択することで、セルの追加などに合わせてテキストボックスの位置が調整されます。）

Excelの「データの分析機能」でも注などをテーブルの中に埋め込んだ場合、これらも対象のデータとして取り込もうとしているようです。

openpyxlを用いて、ワークシートを読み込んだ場合、プログラム側でテーブルの構造を意識して、
明示的にセル位置を指定する必要があります。


```python
print(wb.worksheets, end="\n\n")
for i in range(1,5):
    [print(c.value, end=", ") for c in wb.worksheets[0][f"{i}"]]
    print()
```

    [<Worksheet "総接種回数">, <Worksheet "一般接種">, <Worksheet "医療従事者等">]
    
    これまでのワクチン総接種回数（都道府県別）, None, None, None, None, 
    None, None, None, （8月23日公表時点）, None, 
    都道府県名, 接種回数, 内１回目, 内２回目, None, 
    合計, 112171725, 62773195, 49398530, None, 


余分なデータがあることで、どんな影響がでるか、データをグラフ化して見て見ましょう。


```python
import matplotlib
font = {'family' : "Hiragino Mincho ProN"}
matplotlib.rc('font', **font)
df0.plot.bar(x="都道府県名",y='接種回数',figsize=(16,9))
```




    <AxesSubplot:xlabel='都道府県名'>




    
![png](output_7_1.png)
    


## データの正規化
接種データは本来、各接種記録毎に、'接種場所','接種日時', '接種者の分類：一般、高齢者、医療従事者等','ワクチンの種別','ワクチン接種は1回目か2回目か'といった情報が含まれていると想像される。このファイルには、この生な接種情報を接種日時については集約したあとのデータを集積し、さらに’都道府県毎’にデータを集約していったものとみることができます。　このことは、このエクセルファイルの項目名でのセル結合の様子をみることでも裏付けられます。
このような視点で、できるだけ生データに近い形で、データを再整理したものが次の"接種回数DB"です。


```python
df=pandas.read_excel("kenbetsu-vaccination_data2.xlsx",
                     sheet_name="接種回数DB",
                    index_col=0
                    )
#index_col=(0,1,2,3,4,))
#df=df[['接種地域', 'ワクチン供給者', '接種対象', '接種回','接種済数', 'ワクチン累積供給量']]
#df=df[['接種済数', 'ワクチン累積供給量']]


df.columns, df.index.names
```




    (Index(['接種地域', 'ワクチン供給者', '接種対象', '接種回', '接種済数', 'ワクチン累積供給量'], dtype='object'),
     FrozenList(['id']))



このように、データを再構成することで、簡単にデータをプログラムから読み込むことができます。
また、さまざま集約も標準の機能をつかうことで、簡単に結果をもとめることができます。


```python
df.groupby(["ワクチン供給者"]).sum()
```




<div>
<style scoped>
    .dataframe tbody tr th:only-of-type {
        vertical-align: middle;
    }

    .dataframe tbody tr th {
        vertical-align: top;
    }

    .dataframe thead th {
        text-align: right;
    }
</style>
<table border="1" class="dataframe">
  <thead>
    <tr style="text-align: right;">
      <th></th>
      <th>接種済数</th>
      <th>ワクチン累積供給量</th>
    </tr>
    <tr>
      <th>ワクチン供給者</th>
      <th></th>
      <th></th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th>ファイザー社</th>
      <td>102015703</td>
      <td>124131930.0</td>
    </tr>
    <tr>
      <th>武田/モデルナ社</th>
      <td>10156022</td>
      <td>22232900.0</td>
    </tr>
  </tbody>
</table>
</div>




```python
df.groupby(["ワクチン供給者"]).sum().sum()
```




    id               39903.0
    接種済数         112171725.0
    ワクチン累積供給量    146364830.0
    dtype: float64




```python
df.groupby(["接種地域",'接種回','接種対象']).sum().sort_index(),df.groupby(["接種地域"]).sum().sum()
```




    (                   id     接種済数  ワクチン累積供給量
     接種地域 接種回  接種対象                           
     三重県  内１回目 一般接種     95   785109  1980355.0
               医療従事者等  118    89383        0.0
          内２回目 一般接種    377   634022        0.0
               医療従事者等  259    81345        0.0
     京都府  内１回目 一般接種     99  1096642  2957855.0
     ...               ...      ...        ...
     鳥取県  内２回目 医療従事者等  266    31073        0.0
     鹿児島県 内１回目 一般接種    139   727002  1861520.0
               医療従事者等  140   109133        0.0
          内２回目 一般接種    421   586527        0.0
               医療従事者等  281    95738        0.0
     
     [188 rows x 3 columns],
     id               39903.0
     接種済数         112171725.0
     ワクチン累積供給量    146364830.0
     dtype: float64)




```python
df.groupby(["接種地域"]).sum().sort_index()
```




<div>
<style scoped>
    .dataframe tbody tr th:only-of-type {
        vertical-align: middle;
    }

    .dataframe tbody tr th {
        vertical-align: top;
    }

    .dataframe thead th {
        text-align: right;
    }
</style>
<table border="1" class="dataframe">
  <thead>
    <tr style="text-align: right;">
      <th></th>
      <th>id</th>
      <th>接種済数</th>
      <th>ワクチン累積供給量</th>
    </tr>
    <tr>
      <th>接種地域</th>
      <th></th>
      <th></th>
      <th></th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th>三重県</th>
      <td>849</td>
      <td>1589859</td>
      <td>1980355.0</td>
    </tr>
    <tr>
      <th>京都府</th>
      <td>861</td>
      <td>2238427</td>
      <td>2957855.0</td>
    </tr>
    <tr>
      <th>佐賀県</th>
      <td>951</td>
      <td>845812</td>
      <td>985815.0</td>
    </tr>
    <tr>
      <th>兵庫県</th>
      <td>873</td>
      <td>4957700</td>
      <td>6100055.0</td>
    </tr>
    <tr>
      <th>北海道</th>
      <td>711</td>
      <td>4628368</td>
      <td>5979910.0</td>
    </tr>
    <tr>
      <th>千葉県</th>
      <td>777</td>
      <td>5200353</td>
      <td>6107845.0</td>
    </tr>
    <tr>
      <th>和歌山県</th>
      <td>885</td>
      <td>1021734</td>
      <td>1196545.0</td>
    </tr>
    <tr>
      <th>埼玉県</th>
      <td>771</td>
      <td>5768529</td>
      <td>6801570.0</td>
    </tr>
    <tr>
      <th>大分県</th>
      <td>969</td>
      <td>1106544</td>
      <td>1270095.0</td>
    </tr>
    <tr>
      <th>大阪府</th>
      <td>867</td>
      <td>7458806</td>
      <td>10757385.0</td>
    </tr>
    <tr>
      <th>奈良県</th>
      <td>879</td>
      <td>1266858</td>
      <td>1577330.0</td>
    </tr>
    <tr>
      <th>宮城県</th>
      <td>729</td>
      <td>2039998</td>
      <td>2579595.0</td>
    </tr>
    <tr>
      <th>宮崎県</th>
      <td>975</td>
      <td>993101</td>
      <td>1155855.0</td>
    </tr>
    <tr>
      <th>富山県</th>
      <td>801</td>
      <td>952505</td>
      <td>1135070.0</td>
    </tr>
    <tr>
      <th>山口県</th>
      <td>915</td>
      <td>1545104</td>
      <td>1836535.0</td>
    </tr>
    <tr>
      <th>山形県</th>
      <td>741</td>
      <td>1125335</td>
      <td>1365055.0</td>
    </tr>
    <tr>
      <th>山梨県</th>
      <td>819</td>
      <td>692941</td>
      <td>951420.0</td>
    </tr>
    <tr>
      <th>岐阜県</th>
      <td>831</td>
      <td>1923575</td>
      <td>2306565.0</td>
    </tr>
    <tr>
      <th>岡山県</th>
      <td>903</td>
      <td>1869401</td>
      <td>2246580.0</td>
    </tr>
    <tr>
      <th>岩手県</th>
      <td>723</td>
      <td>1083575</td>
      <td>1353435.0</td>
    </tr>
    <tr>
      <th>島根県</th>
      <td>897</td>
      <td>658635</td>
      <td>745740.0</td>
    </tr>
    <tr>
      <th>広島県</th>
      <td>909</td>
      <td>2550235</td>
      <td>3218420.0</td>
    </tr>
    <tr>
      <th>徳島県</th>
      <td>921</td>
      <td>732888</td>
      <td>917095.0</td>
    </tr>
    <tr>
      <th>愛媛県</th>
      <td>933</td>
      <td>1250690</td>
      <td>1556050.0</td>
    </tr>
    <tr>
      <th>愛知県</th>
      <td>843</td>
      <td>6200562</td>
      <td>8725620.0</td>
    </tr>
    <tr>
      <th>新潟県</th>
      <td>795</td>
      <td>2146330</td>
      <td>2686645.0</td>
    </tr>
    <tr>
      <th>東京都</th>
      <td>783</td>
      <td>12134011</td>
      <td>20893610.0</td>
    </tr>
    <tr>
      <th>栃木県</th>
      <td>759</td>
      <td>1502088</td>
      <td>1966330.0</td>
    </tr>
    <tr>
      <th>沖縄県</th>
      <td>987</td>
      <td>1119842</td>
      <td>1406405.0</td>
    </tr>
    <tr>
      <th>滋賀県</th>
      <td>855</td>
      <td>1230505</td>
      <td>1575270.0</td>
    </tr>
    <tr>
      <th>熊本県</th>
      <td>963</td>
      <td>1878851</td>
      <td>2204255.0</td>
    </tr>
    <tr>
      <th>石川県</th>
      <td>807</td>
      <td>1108983</td>
      <td>1349610.0</td>
    </tr>
    <tr>
      <th>神奈川県</th>
      <td>789</td>
      <td>7424780</td>
      <td>9002475.0</td>
    </tr>
    <tr>
      <th>福井県</th>
      <td>813</td>
      <td>773434</td>
      <td>939520.0</td>
    </tr>
    <tr>
      <th>福岡県</th>
      <td>945</td>
      <td>4647952</td>
      <td>5684120.0</td>
    </tr>
    <tr>
      <th>福島県</th>
      <td>747</td>
      <td>1786718</td>
      <td>2106990.0</td>
    </tr>
    <tr>
      <th>秋田県</th>
      <td>735</td>
      <td>980253</td>
      <td>1106205.0</td>
    </tr>
    <tr>
      <th>群馬県</th>
      <td>765</td>
      <td>2008977</td>
      <td>2599545.0</td>
    </tr>
    <tr>
      <th>茨城県</th>
      <td>753</td>
      <td>2569535</td>
      <td>3273575.0</td>
    </tr>
    <tr>
      <th>長崎県</th>
      <td>957</td>
      <td>1396867</td>
      <td>1654190.0</td>
    </tr>
    <tr>
      <th>長野県</th>
      <td>825</td>
      <td>1867238</td>
      <td>2292470.0</td>
    </tr>
    <tr>
      <th>青森県</th>
      <td>717</td>
      <td>1198827</td>
      <td>1428815.0</td>
    </tr>
    <tr>
      <th>静岡県</th>
      <td>837</td>
      <td>3060754</td>
      <td>3901980.0</td>
    </tr>
    <tr>
      <th>香川県</th>
      <td>927</td>
      <td>841707</td>
      <td>1097190.0</td>
    </tr>
    <tr>
      <th>高知県</th>
      <td>939</td>
      <td>719429</td>
      <td>874865.0</td>
    </tr>
    <tr>
      <th>鳥取県</th>
      <td>891</td>
      <td>554709</td>
      <td>651450.0</td>
    </tr>
    <tr>
      <th>鹿児島県</th>
      <td>981</td>
      <td>1518400</td>
      <td>1861520.0</td>
    </tr>
  </tbody>
</table>
</div>




```python
df.groupby(["接種地域"]).sum().sum()
```




    id               39903.0
    接種済数         112171725.0
    ワクチン累積供給量    146364830.0
    dtype: float64



## 結論

ここで試みたことの結論は、

- データ項目名のセル結合は、その表が(RDB的な意味で）正規化されていないことの具体的な表れであること。
- その構造を理解することで、データの正規化をおこなうことで、プログラム処理が容易になる。

とまとめられるでしょう。また、

- excelファイル作成時には、（正規化された）データだけのテーブルを含むworksheetと,ユーザーとの入出力のためのworksheetを分けたデザインを使う。

ことが望ましいでしょう。

テーブルの「正規化」を、もう少しわかり易い言葉遣いで説明できるように工夫すべきなのかもしれません。

ヘッダ／フッターやテキストボックスを利用することで、データに直接関係のないデータはテーブル（セル）に書き込まないようにできるということの啓蒙も必要でしょう。


```python
df[(df["接種地域"]=="北海道")&(df['ワクチン供給者'] =="ファイザー社")].groupby("接種回").sum()
```




<div>
<style scoped>
    .dataframe tbody tr th:only-of-type {
        vertical-align: middle;
    }

    .dataframe tbody tr th {
        vertical-align: top;
    }

    .dataframe thead th {
        text-align: right;
    }
</style>
<table border="1" class="dataframe">
  <thead>
    <tr style="text-align: right;">
      <th></th>
      <th>id</th>
      <th>接種済数</th>
      <th>ワクチン累積供給量</th>
    </tr>
    <tr>
      <th>接種回</th>
      <th></th>
      <th></th>
      <th></th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th>内１回目</th>
      <td>96</td>
      <td>2320702</td>
      <td>5198310.0</td>
    </tr>
    <tr>
      <th>内２回目</th>
      <td>378</td>
      <td>1956696</td>
      <td>0.0</td>
    </tr>
  </tbody>
</table>
</div>




```python
df[(df["接種地域"] == "北海道")&(df['ワクチン供給者'] == "ファイザー社")]
```




<div>
<style scoped>
    .dataframe tbody tr th:only-of-type {
        vertical-align: middle;
    }

    .dataframe tbody tr th {
        vertical-align: top;
    }

    .dataframe thead th {
        text-align: right;
    }
</style>
<table border="1" class="dataframe">
  <thead>
    <tr style="text-align: right;">
      <th></th>
      <th>接種地域</th>
      <th>ワクチン供給者</th>
      <th>接種対象</th>
      <th>接種回</th>
      <th>接種済数</th>
      <th>ワクチン累積供給量</th>
    </tr>
    <tr>
      <th>id</th>
      <th></th>
      <th></th>
      <th></th>
      <th></th>
      <th></th>
      <th></th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th>1</th>
      <td>北海道</td>
      <td>ファイザー社</td>
      <td>一般接種</td>
      <td>内１回目</td>
      <td>1991581</td>
      <td>5198310.0</td>
    </tr>
    <tr>
      <th>95</th>
      <td>北海道</td>
      <td>ファイザー社</td>
      <td>医療従事者等</td>
      <td>内１回目</td>
      <td>329121</td>
      <td>NaN</td>
    </tr>
    <tr>
      <th>142</th>
      <td>北海道</td>
      <td>ファイザー社</td>
      <td>一般接種</td>
      <td>内２回目</td>
      <td>1663807</td>
      <td>NaN</td>
    </tr>
    <tr>
      <th>236</th>
      <td>北海道</td>
      <td>ファイザー社</td>
      <td>医療従事者等</td>
      <td>内２回目</td>
      <td>292889</td>
      <td>NaN</td>
    </tr>
  </tbody>
</table>
</div>




```python
df[(df["接種地域"].isin(["北海道","青森県"]))&(df['ワクチン供給者'] == "ファイザー社")]
```




<div>
<style scoped>
    .dataframe tbody tr th:only-of-type {
        vertical-align: middle;
    }

    .dataframe tbody tr th {
        vertical-align: top;
    }

    .dataframe thead th {
        text-align: right;
    }
</style>
<table border="1" class="dataframe">
  <thead>
    <tr style="text-align: right;">
      <th></th>
      <th>接種地域</th>
      <th>ワクチン供給者</th>
      <th>接種対象</th>
      <th>接種回</th>
      <th>接種済数</th>
      <th>ワクチン累積供給量</th>
    </tr>
    <tr>
      <th>id</th>
      <th></th>
      <th></th>
      <th></th>
      <th></th>
      <th></th>
      <th></th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th>1</th>
      <td>北海道</td>
      <td>ファイザー社</td>
      <td>一般接種</td>
      <td>内１回目</td>
      <td>1991581</td>
      <td>5198310.0</td>
    </tr>
    <tr>
      <th>2</th>
      <td>青森県</td>
      <td>ファイザー社</td>
      <td>一般接種</td>
      <td>内１回目</td>
      <td>583289</td>
      <td>1383915.0</td>
    </tr>
    <tr>
      <th>95</th>
      <td>北海道</td>
      <td>ファイザー社</td>
      <td>医療従事者等</td>
      <td>内１回目</td>
      <td>329121</td>
      <td>NaN</td>
    </tr>
    <tr>
      <th>96</th>
      <td>青森県</td>
      <td>ファイザー社</td>
      <td>医療従事者等</td>
      <td>内１回目</td>
      <td>67672</td>
      <td>NaN</td>
    </tr>
    <tr>
      <th>142</th>
      <td>北海道</td>
      <td>ファイザー社</td>
      <td>一般接種</td>
      <td>内２回目</td>
      <td>1663807</td>
      <td>NaN</td>
    </tr>
    <tr>
      <th>143</th>
      <td>青森県</td>
      <td>ファイザー社</td>
      <td>一般接種</td>
      <td>内２回目</td>
      <td>471657</td>
      <td>NaN</td>
    </tr>
    <tr>
      <th>236</th>
      <td>北海道</td>
      <td>ファイザー社</td>
      <td>医療従事者等</td>
      <td>内２回目</td>
      <td>292889</td>
      <td>NaN</td>
    </tr>
    <tr>
      <th>237</th>
      <td>青森県</td>
      <td>ファイザー社</td>
      <td>医療従事者等</td>
      <td>内２回目</td>
      <td>59963</td>
      <td>NaN</td>
    </tr>
  </tbody>
</table>
</div>




```python
ef=pandas.ExcelFile("kenbetsu-vaccination_data2.xlsx")
df1=ef.parse(sheet_name="接種回数DB",
    index_col=(0,1,2,3,4,))
# equivalent to:
#df1=pandas.read_excel("kenbetsu-vaccination_data2.xlsx",
#                     sheet_name="接種回数DB",
#    index_col=(0,1,2,3,4,))
df1.columns
```




    Index(['接種済数', 'ワクチン累積供給量'], dtype='object')




```python
df1.index
```




    MultiIndex([(  1,  '北海道', 'ファイザー社',   '一般接種', '内１回目'),
                (  2,  '青森県', 'ファイザー社',   '一般接種', '内１回目'),
                (  3,  '岩手県', 'ファイザー社',   '一般接種', '内１回目'),
                (  4,  '宮城県', 'ファイザー社',   '一般接種', '内１回目'),
                (  5,  '秋田県', 'ファイザー社',   '一般接種', '内１回目'),
                (  6,  '山形県', 'ファイザー社',   '一般接種', '内１回目'),
                (  7,  '福島県', 'ファイザー社',   '一般接種', '内１回目'),
                (  8,  '茨城県', 'ファイザー社',   '一般接種', '内１回目'),
                (  9,  '栃木県', 'ファイザー社',   '一般接種', '内１回目'),
                ( 10,  '群馬県', 'ファイザー社',   '一般接種', '内１回目'),
                ...
                (273,  '愛媛県', 'ファイザー社', '医療従事者等', '内２回目'),
                (274,  '高知県', 'ファイザー社', '医療従事者等', '内２回目'),
                (275,  '福岡県', 'ファイザー社', '医療従事者等', '内２回目'),
                (276,  '佐賀県', 'ファイザー社', '医療従事者等', '内２回目'),
                (277,  '長崎県', 'ファイザー社', '医療従事者等', '内２回目'),
                (278,  '熊本県', 'ファイザー社', '医療従事者等', '内２回目'),
                (279,  '大分県', 'ファイザー社', '医療従事者等', '内２回目'),
                (280,  '宮崎県', 'ファイザー社', '医療従事者等', '内２回目'),
                (281, '鹿児島県', 'ファイザー社', '医療従事者等', '内２回目'),
                (282,  '沖縄県', 'ファイザー社', '医療従事者等', '内２回目')],
               names=['id', '接種地域', 'ワクチン供給者', '接種対象', '接種回'], length=282)



MultiIndexに対しては、`isin`メソッドを使って条件を記述する。


```python
df1[df1.index.isin(["青森県","東京都"], level="接種地域") 
   & df1.index.isin(["ファイザー社"], level="ワクチン供給者")].groupby(["接種地域","接種対象","接種回"]).sum()
```




<div>
<style scoped>
    .dataframe tbody tr th:only-of-type {
        vertical-align: middle;
    }

    .dataframe tbody tr th {
        vertical-align: top;
    }

    .dataframe thead th {
        text-align: right;
    }
</style>
<table border="1" class="dataframe">
  <thead>
    <tr style="text-align: right;">
      <th></th>
      <th></th>
      <th></th>
      <th>接種済数</th>
      <th>ワクチン累積供給量</th>
    </tr>
    <tr>
      <th>接種地域</th>
      <th>接種対象</th>
      <th>接種回</th>
      <th></th>
      <th></th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th rowspan="4" valign="top">東京都</th>
      <th rowspan="2" valign="top">一般接種</th>
      <th>内１回目</th>
      <td>5117917</td>
      <td>13310310.0</td>
    </tr>
    <tr>
      <th>内２回目</th>
      <td>3877286</td>
      <td>0.0</td>
    </tr>
    <tr>
      <th rowspan="2" valign="top">医療従事者等</th>
      <th>内１回目</th>
      <td>610484</td>
      <td>0.0</td>
    </tr>
    <tr>
      <th>内２回目</th>
      <td>545945</td>
      <td>0.0</td>
    </tr>
    <tr>
      <th rowspan="4" valign="top">青森県</th>
      <th rowspan="2" valign="top">一般接種</th>
      <th>内１回目</th>
      <td>583289</td>
      <td>1383915.0</td>
    </tr>
    <tr>
      <th>内２回目</th>
      <td>471657</td>
      <td>0.0</td>
    </tr>
    <tr>
      <th rowspan="2" valign="top">医療従事者等</th>
      <th>内１回目</th>
      <td>67672</td>
      <td>0.0</td>
    </tr>
    <tr>
      <th>内２回目</th>
      <td>59963</td>
      <td>0.0</td>
    </tr>
  </tbody>
</table>
</div>



`lambda`式をつかえば、より柔軟に記述できるが、記述量、読みやすさの観点からは、「最後の武器　」と考えるのが良いでしょう。


```python
df1[[(lambda x:(x[1] == "北海道")
      &(x[2] == "ファイザー社"))(i)
     for i in df1.index]
   ]
```




<div>
<style scoped>
    .dataframe tbody tr th:only-of-type {
        vertical-align: middle;
    }

    .dataframe tbody tr th {
        vertical-align: top;
    }

    .dataframe thead th {
        text-align: right;
    }
</style>
<table border="1" class="dataframe">
  <thead>
    <tr style="text-align: right;">
      <th></th>
      <th></th>
      <th></th>
      <th></th>
      <th></th>
      <th>接種済数</th>
      <th>ワクチン累積供給量</th>
    </tr>
    <tr>
      <th>id</th>
      <th>接種地域</th>
      <th>ワクチン供給者</th>
      <th>接種対象</th>
      <th>接種回</th>
      <th></th>
      <th></th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th>1</th>
      <th>北海道</th>
      <th>ファイザー社</th>
      <th>一般接種</th>
      <th>内１回目</th>
      <td>1991581</td>
      <td>5198310.0</td>
    </tr>
    <tr>
      <th>95</th>
      <th>北海道</th>
      <th>ファイザー社</th>
      <th>医療従事者等</th>
      <th>内１回目</th>
      <td>329121</td>
      <td>NaN</td>
    </tr>
    <tr>
      <th>142</th>
      <th>北海道</th>
      <th>ファイザー社</th>
      <th>一般接種</th>
      <th>内２回目</th>
      <td>1663807</td>
      <td>NaN</td>
    </tr>
    <tr>
      <th>236</th>
      <th>北海道</th>
      <th>ファイザー社</th>
      <th>医療従事者等</th>
      <th>内２回目</th>
      <td>292889</td>
      <td>NaN</td>
    </tr>
  </tbody>
</table>
</div>




```python
df1[[(lambda x:(x[1] in ("北海道","青森県")) &
      (x[2] == "ファイザー社"))(i) for i in df1.index]
   ].groupby(["接種地域","接種対象","接種回"]).sum()
```




<div>
<style scoped>
    .dataframe tbody tr th:only-of-type {
        vertical-align: middle;
    }

    .dataframe tbody tr th {
        vertical-align: top;
    }

    .dataframe thead th {
        text-align: right;
    }
</style>
<table border="1" class="dataframe">
  <thead>
    <tr style="text-align: right;">
      <th></th>
      <th></th>
      <th></th>
      <th>接種済数</th>
      <th>ワクチン累積供給量</th>
    </tr>
    <tr>
      <th>接種地域</th>
      <th>接種対象</th>
      <th>接種回</th>
      <th></th>
      <th></th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th rowspan="4" valign="top">北海道</th>
      <th rowspan="2" valign="top">一般接種</th>
      <th>内１回目</th>
      <td>1991581</td>
      <td>5198310.0</td>
    </tr>
    <tr>
      <th>内２回目</th>
      <td>1663807</td>
      <td>0.0</td>
    </tr>
    <tr>
      <th rowspan="2" valign="top">医療従事者等</th>
      <th>内１回目</th>
      <td>329121</td>
      <td>0.0</td>
    </tr>
    <tr>
      <th>内２回目</th>
      <td>292889</td>
      <td>0.0</td>
    </tr>
    <tr>
      <th rowspan="4" valign="top">青森県</th>
      <th rowspan="2" valign="top">一般接種</th>
      <th>内１回目</th>
      <td>583289</td>
      <td>1383915.0</td>
    </tr>
    <tr>
      <th>内２回目</th>
      <td>471657</td>
      <td>0.0</td>
    </tr>
    <tr>
      <th rowspan="2" valign="top">医療従事者等</th>
      <th>内１回目</th>
      <td>67672</td>
      <td>0.0</td>
    </tr>
    <tr>
      <th>内２回目</th>
      <td>59963</td>
      <td>0.0</td>
    </tr>
  </tbody>
</table>
</div>


